#ifndef _INCLUDE_aimms_Project
#define _INCLUDE_aimms_Project

#include <string>
#include <set>

namespace aimms{

    class ISession;
    class IData;
    class IDataView;
    class IMultiDimData;
    class ISetData;
    class CallBack;

    class Project
    {

    public:
        Project();
        ~Project();

        const char* getProjectName() const;

    public:
        //Establishes the connection with AIMMS and opens the project.
        int open(const char* szAimmsLocation, const char* ProjectName, int timeOut, int maxFlushInBackground, int maxMemoryFlushInBackground, int maxBufferSize, class ErrorCollector* ec);

        //Retrieves the set from the AIMMS project.
        //Adds the labels to the set.
        //if clearfirst==true, the set is emptied in advance.
        int assignSet(const char* name, int card, const char* const* labels, bool clearFirst);

        //Retrieves the identifier from the AIMMS project.
        //Verifies this is a double valued parameter and the dimension corresponds with the number of sets.
        //Assigns the data to the positions given by sets, clears all data outside the sets
        //Assumes a column major ordering. Applies to identifiers of which all index domains are subsets of Integers.
        template <typename T>
        int assignIntIdentifier(const char* name, int dim, int* firsts, int* lasts, T* data, int** slices=0, int* slicecards=0);
        //Idem, but uses the ordinals of the index domains. No restriction on the index domains.
        template<typename T>
        int assignOrdIdentifier(const char* name, int dim, int* cards, T* data, int** slices=0, int* slicecards=0);
        
        //Runs the AIMMS procedure
        int runProcedure(const char* name, int& returncode);

        //Retrieves the identifier from the AIMMS project.
        //Verifies this is a double valued parameter and the dimension corresponds with the number of sets.
        //Retrieves the data to the positions given by sets,  assuming a column major ordering. Applies to identifiers of which all index domains are subsets of Integers.
        template<typename T>
        int retrieveIntIdentifier(const char* name, int dim, int* firsts, int* lasts, T* data, int** slices=0, int* slicecards=0);
        //Idem, but uses the ordinals of the index domains. No restriction on the index domains.
        template<typename T>
        int retrieveOrdIdentifier(const char* name, int dim, int* cards, T* data, int** slices=0, int* slicecards=0);
                
        //Closes the project and the connection with AIMMS.
        int close();

    private:
        int getIdentifier(const char* name, int dim, IData*& ret);
        template<typename T>
        int retrieveScalar(IData* scalar, T* data);
        template<typename T>
        int assignScalar(IData* scalar, T* data);
        bool cardsOK(int* cards, IMultiDimData* par) ;
        void cacheIndexDomains( IDataView* par);
        template<typename T>
        void setValuesCMO(IDataView* par, T* data, int dim, int* firsts, int* lasts);
        template<typename T>
        void setValuesCMO(IDataView* par, T* data, int dim, int* cards);
    private:
        std::string m_strProjectName;
        ISession* m_Session;
        CallBack* m_CallBack;
        typedef std::set<const ISetData*>  setOfISetP;
        setOfISetP m_SetCache;
        
    };

}


#endif

